C Converts database format to Cornell Condensed Format
      parameter(mm=10000,nn=10000,nndat=2000000)
C mm is max number of samples; nn of species
C nndat is size of main data matrix
      character*30 ddji(nndat),dji
      character*20 filein,fileou
      character*10 samp,samp1,spec,blnk10/'          '/,word
      character*10 sa(mm),sp(nn),d(3),weight
      character*80 blurb,blnk80
      character*1 b(80),blank/' '/,w(10),one/'1'/,ww
      integer iddji(nndat),jout(80),jbeg(nn),jend(nn),jtot(nn)
      real wddji(nndat),wout(80)
      equivalence (b,blurb),(w,word),(d,dji)
C these equivalences are used in reading in the data and writing out filename

      do k=1,80
         b(k)=blank
         enddo
      blnk80=blurb
      samp1=blnk10

      write(*,2100) mm,nn,nndat
 2100 format(//1x,'DATACORN - converts database to Cornell format'/
     1       1x,'written by Mark Hill, October 2011'//
     2       1x,'Input limits:'/
     3       4x,'Max number of samples = ', I7 /
     4       4x,'Max number of species = ', I7 /
     5       4x,'Max length of data array = ', I7 /)
      write(*,2105)
 2105 format(1x,'Type name of input file [sample species quant] ...')
      call filin(filein,4)
      write(*,2106)
 2106 format(1x,'Type name of output file ...')
      call filout(fileou,7)
      iconv=0
      write(*,2107)
 2107 format(1x,'Type 1 for removal of underscores from species names'/
     1       1x,'                e.g. GGGG_SSSS converted to GGGGSSSS'/
     2       1x,'or press <RETURN> to use first 8 characters of names')
      word=blnk10
      read(*,2108) word
 2108 format(a10)
      do k=1,10
         if(w(k).eq.one) iconv=1
         enddo

C Read in data to ddij, setting up indexes sa(m),sp(n)
      call readin(samp,spec,weight,iend,blurb,blnk80,b,sa,sp,m,n,d,
     1         ndji,word,blnk10,w,ddji,dji,nndat,jbeg,jtot,jend,mm,nn)

C Recode to replace characters by integers for samples and reals for weights
      call recode(dji,d,ddji,iddji,wddji,ndji,sp,n,word,w)

C Check for duplicated entries
      do jj=1,m
         ii=0
         do idji=jbeg(jj),jend(jj)
            i=iddji(idji)
            if(i.eq.ii) then
               write(*,2990) sa(jj),sp(i)
 2990 format(/1x,'*** ERROR *** Repeated data couplet ',a10,1x,a10/
     1        1x,'              Reprocess your data to remove repeats')
               call hold
               stop
               end if
            ii=i             
            enddo
         enddo

C Write out result
      blurb=blnk80
      write(blurb,2002) fileou,filein
 2002 format(a20,' converted from ',a20)
      ww=blank
      kkk=0      
      do kk=1,80
         if(b(kk).eq.blank.and.ww.eq.blank) goto 150
         kkk=kkk+1
         b(kkk)=b(kk)
         ww=b(kk)
  150    continue
         enddo
      do k=kkk+1,80
         b(k)=blank
         enddo
      write(7,2005) blurb
 2005 format(a80)
      write(7,2006)
 2006 format('(i5,2x,6(i5,f7.1))',51x,'6')
      do jj=1,m
         k=1+(jend(jj)-jbeg(jj))/6
         do kk=1,k
            k1=6*kk-5
            k2=min(6,jend(jj)-k1-jbeg(jj)+2)
            do kkk=1,k2
               jout(kkk)=iddji(jbeg(jj)+kkk+k1-2)
               wout(kkk)=wddji(jbeg(jj)+kkk+k1-2)
               enddo
            write(7,2000) jj,(jout(kkk),wout(kkk),kkk=1,k2)
 2000 format(i5,2x,6(i5,f7.1))
            enddo
         enddo
      write(7,2000) 0
C If conversion to remove underscores is required, now do it
      if(iconv.eq.1) then
         do j=1,n
            spec=sp(j)
            call conv(spec,word,w)
            sp(j)=spec
            enddo
         end if
      write(7,2001) (sp(j),j=1,n)
      write(7,2001) (sa(i),i=1,m)
 2001 format(10a8)         
      call hold
      stop
      end

      subroutine conv(spec,word,w)
C Converts from form gggg_ssss to ggggssss - the standard Cornell form
      character*10 spec,word,blank/' '/
      character*1 w(10)
      word=spec
      do k=6,9
         w(k-1)=w(k)
         enddo
      w(9)=blank
      spec=word
      return
      end

      subroutine readin(samp,spec,weight,iend,blurb,blnk80,b,sa,sp,m,n,
     1    d,ndji,word,blnk10,w,ddji,dji,nndat,jbeg,jtot,jend,mm,nn)
      character*30 ddji(nndat),dji
      character*10 samp,samp1,spec,word
      character*10 sa(mm),sp(nn),d(3),weight,blnk10
      character*80 blurb,blnk80
      character*1 b(80),w(10)
      integer jtot(mm),jbeg(mm),jend(mm)
C Read in data using subroutine getd
      m=0
      n=0
      ndji=0
      iidji=0
      
  100 continue
      call getd(4,samp,spec,weight,iend,blurb,blnk80,b,word,blnk10,w)
      if(iend.eq.1.and.blurb.eq.blnk80) goto 101
      if(blurb.eq.blnk80) goto 100
      call addwrd(sa,mm,m,samp)
      call addwrd(sp,nn,n,spec)
      ndji=ndji+1
      if(mod(ndji,10000).eq.0) write(*,*) samp, spec, weight, ndji
      d(1)=samp
      d(2)=spec
      d(3)=weight
      ddji(ndji)=dji
      if(iend.eq.1) goto 101
      goto 100

  101 continue
         write(*,2000) m,n,ndji
 2000 format(/4x,'Number of samples    = ', I7 /
     1        4x,'Number of species    = ', I7 /
     2        4x,'Length of data array = ', I7 /)
      call sort30(ddji,ndji)
C The main dataset is sorted and in ddji; next to calculate numbers of
C samples per species

C Now calculate the number of data items for each species in main data
      write(*,*) ' Sort complete'
C Set jtot to 0
      do j=1,m
         jtot(j)=0
         enddo
      samp1=blnk10
      do idji=1,ndji
         dji=ddji(idji)
         samp=d(1)
         spec=d(2)
         call binfnd(sa,m,samp,j)
         jtot(j)=jtot(j)+1
         if(samp.ne.samp1) samp1=samp
         enddo
      do j=1,m
         if(j.ne.1) then
            jbeg(j)=jbeg(j-1)+jtot(j-1)
            jend(j)=jend(j-1)+jtot(j)
            else
            jbeg(j)=1
            jend(j)=jtot(j)
            end if
         enddo
      return
      end

      subroutine recode(dji,d,ddji,iddji,wddji,ndji,sa,m,word,w)
C Recode the data, replacing samp by index in sa and weight by real
C wgt is set to 1 if all entries are blank
      character*30 ddji(ndji),dji
      character*10 samp,word,sa(m),d(3),weight
      character*1 w(10)
      real wddji(ndji)
      integer iddji(ndji)
      small=1.0E-18
      wgttot=0
      do idji=1,ndji
         dji=ddji(idji)
         samp=d(2)
         call binfnd(sa,m,samp,i)
         iddji(idji)=i        
         weight=d(3)
         call getnum(weight,wgt,word,w)
         if(wgt.eq.0) wgt=small
         wddji(idji)=wgt
         wgttot=wgttot+wgt
         enddo
      if(wgttot.lt.1.0E-10) then
         do idji=1,ndji
            wddji(idji)=1
            enddo
         write(*,2000)
 2000 format(1x,' *** No quantities in data; ',
     1          'values for all occurrences have been set to 1 ***')
         end if
      return
      end

      subroutine sort10(dict,n)
C To sort character*10 entities
      character*10 dict(n),djx,djjx,djjjx
      integer i,j,jj,jjj
      do 10 i=1,n
      j=i
      djx=dict(j)
    5 if(j.eq.1) goto 8
      jj=j/2
      djjx=dict(jj)
      if(djjx.gt.djx) goto 8
      if(djjx.eq.djx) then
           end if
      dict(j)=djjx
      j=jj
      goto 5
    8 dict(j)=djx
   10 continue
      i=n
      goto 14
   12 dict(j)=djx
   14 if(i.eq.1) return
      djx=dict(i)
      dict(i)=dict(1)
      i=i-1
      j=1
      jj=2
   15 if(i-jj) 12,17,16
   16 djjx=dict(jj)
      jjj=jj+1
      djjjx=dict(jjj)
      if(djjx.ge.djjjx) goto 18
      if(djx.ge.djjjx) goto 12
      dict(j)=djjjx
      j=jjj
      jj=j*2
      goto 15
   17 djjx=dict(jj)
   18 if(djx.ge.djjx) goto 12
      dict(j)=djjx
      j=jj
      jj=j*2
      goto 15
      end

      subroutine sort30(dict,n)
C To sort character*30 entities
      character*30 dict(n),djx,djjx,djjjx
      integer i,j,jj,jjj
      do 10 i=1,n
      j=i
      djx=dict(j)
    5 if(j.eq.1) goto 8
      jj=j/2
      djjx=dict(jj)
      if(djjx.gt.djx) goto 8
      if(djjx.eq.djx) then
           end if
      dict(j)=djjx
      j=jj
      goto 5
    8 dict(j)=djx
   10 continue
      i=n
      goto 14
   12 dict(j)=djx
   14 if(i.eq.1) return
      djx=dict(i)
      dict(i)=dict(1)
      i=i-1
      j=1
      jj=2
   15 if(i-jj) 12,17,16
   16 djjx=dict(jj)
      jjj=jj+1
      djjjx=dict(jjj)
      if(djjx.ge.djjjx) goto 18
      if(djx.ge.djjjx) goto 12
      dict(j)=djjjx
      j=jjj
      jj=j*2
      goto 15
   17 djjx=dict(jj)
   18 if(djx.ge.djjx) goto 12
      dict(j)=djjx
      j=jj
      jj=j*2
      goto 15
      end

      subroutine hold
C Asks for user input before exiting, to retain DOS window
      character*1 ch
      write(*,2000)
 2000 format(////'*** Program finished  ***'///
     1       'Press <RETURN> to exit'///
     2              '*************************')
      read(*,1000) ch
 1000 format(a1)
      return
      end

      subroutine filout(filein,iunit)
      character*20 filein
C Checks whether filein available for writing or already exists
  100 continue
      read(*,1000) filein
 1000 format(a20)
      open(unit=iunit,file=filein,status='new',err=999)
      return
  999 continue
      write(*,2000)
 2000 format(/'  *** ERROR *** File already exists'/
     1        ' Type another name')
      goto 100
      end

      subroutine filin(filein,iunit)
      character*20 filein
C Checks whether filein available for reading
  100 continue
      read(*,1000) filein
 1000 format(a20)
      open(unit=iunit,file=filein,status='old',err=999)
      return
  999 continue
      write(*,2000)
 2000 format(/'  *** ERROR *** File does not exist'/
     1        ' Type another name')
      goto 100
      end

      subroutine addwrd(sa,mm,m,samp)
C checks samp against a list of words sa(mm), and adds samp to the list if new
      character*10 sa(mm),samp
      if(m.eq.0) then
         i=0
         else
         call binfnd(sa,m,samp,i)
         end if
      if(i.ne.0) return 
      m=m+1
      sa(m)=samp
      call sort10(sa,m)
      return
      end

        subroutine binfnd(ma,n,na,i)
C Searches for item na in array ma (length n), index of na is i
C i=0 if na cannot be found
        character*10 ma(n),na,iamin,iamid,iamax
        i=0
        imin=1
        iamin=ma(imin)
        imax=n
        iamax=ma(imax)
   10   continue
        if(imax-imin.le.1) then
                if(iamin.eq.na) then
                        i=imin
                        return
                        end if
                if(iamax.eq.na) i=imax
                return
                end if
        imid=(imax+imin)/2
        iamid=ma(imid)
        if(na.le.iamid) then
                imax=imid
                iamax=iamid
                goto 10
                end if
        imin=imid
        iamin=iamid
        goto 10
        end

      subroutine getd(iunit,samp,spec,weight,iend,
     1                  blurb,blnk80,b,word,blnk10,w)
C Gets data from iunit, makes first 3 words: samp spec weight
C iend is set to 0 unless we are at end of file, when it is 1
      character*10 samp,spec,word,weight,blnk10
      character*80 blurb,blnk80
      character*1 b(80),blank/' '/,w(10)
      iend=0
      blurb=blnk80
      read(iunit,1000,end=97) blurb
 1000 format(a80)
      goto 98
   97 iend=1
   98 continue
      if(blurb.eq.blnk80) return
C if blurb is blank then we have no data to add
      do k=1,80
         if(b(k).ne.blank) goto 100
         enddo
      return
  100 continue
      k1=k
      do k=k1,80
         if(b(k).eq.blank) goto 200
         enddo
      return
  200 continue
      k2=k-1
      do k=k2+2,80
         if(b(k).ne.blank) goto 300
         enddo
      return
  300 continue
      k3=k
      do k=k3,80
         if(b(k).eq.blank) goto 400
         enddo
      return
  400 continue
      k4=k-1
      do k=k4+2,80
         if(b(k).ne.blank) goto 500
         enddo
      if(k.gt.79) k=80
  500 continue
      k5=k
      do k=k5,80
         if(b(k).eq.blank) goto 600
         enddo
  600 continue
      k6=k-1
      if(k5.eq.80) k6=80
      word=blnk10
      do k=k1,k2
         if(k.lt.k1+10) w(1+k-k1)=b(k)
         enddo
      samp=word
      word=blnk10
      do k=k3,k4
         if(k.lt.k3+10) w(1+k-k3)=b(k)
         enddo
      spec=word
      word=blnk10
      do k=k5,k6
         if(k.lt.k5+9) w(1+k-k5)=b(k)
         enddo
      weight=word
      return
      end

      subroutine getnum(weight,wgt,word,w)
C Given a number weight, reads wgt as a real number
      character*10 weight,word,blnk10/'          '/
      character*1 w(10),dot/'.'/,blank/' '/
      real wgt
      if(weight.eq.blnk10) then
         wgt=0
         return
         end if
      word=weight
C Take out leading blanks      
      do k=1,10
         if(w(k).ne.blank) then
            ifirst=k
            goto 50
            end if
         enddo
   50 continue
      do k=1,10
         kk=k+ifirst-1
         if(kk.gt.10) then
            w(k)=blank
            else
            w(k)=w(kk)
            end if
         enddo
C Now look for a dot
      idot=0
      do k=1,10
         if(w(k).eq.dot) idot=1
         if(w(k).eq.blank) goto 100
         enddo
  100 continue
      if(idot.eq.0) w(k)=dot
      read(word,2000,err=200) wgt
 2000 format(f10.6)
      return
  200 continue
      wgt=0
      return
      end
